' LngFnam.bas version 2
'******************************************************************************

$COMPILE        UNIT    'Destination of compilation

'Compiler directives for code generation

$CPU            80386   'Generate code for processor {8086 | 80286 | 80386}
$DIM            ALL     'Declararations required for {ALL | ARRAY | NONE}
$EVENT          OFF     'Generate event checking code
$FLOAT          EMULATE 'FP math package '{EMULATE | NPX | PROCEDURE}
$OPTIMIZE       SPEED   'Optimize code for {SPEED | SIZE}
$OPTION GOSUB   OFF     'Preserve GOSUB stack if a runtime error occurs
$OPTION SIGNED  OFF     'Return addresses by VARSEG etc. as signed integers

'Compiler directives for debugging and testing

$DEBUG UNIT     OFF     'Attach debug information to unit file
$ERROR ALL      OFF     'Generate test code for all types of errors

'******************************************************************************

'----------------------------------------------------------------------
'  Declarations
'----------------------------------------------------------------------

DEFINT A-Z

$INCLUDE        "regnames.inc"
$INCLUDE        "lngfnam2.bi"

' Actions for file handling:
%FILE_CREATE                    =  &h10
%FILE_OPEN                      =     1
%FILE_TRUNCATE                  =     2

DECLARE SUB      FindCLose ()
DECLARE FUNCTION GetDriveNumber (Drive AS STRING) AS BYTE
DECLARE FUNCTION LfnSupported2 () AS INTEGER
DECLARE FUNCTION LfnSupported3 () AS INTEGER

'----------------------------------------------------------------------
' Shared variables
'----------------------------------------------------------------------

DIM DOSError    AS SHARED INTEGER       ' DOS error code

DIM FindHandle  AS SHARED WORD          ' Handle for Win32 file info
DIM FindData    AS SHARED Win32_Find_Data       ' Structure for ditto
DIM PtrFindData AS SHARED Win32_Find_Data PTR   ' and its pointer

DIM DTA         AS SHARED FileInfo PTR  ' pointer to structure for DOS
					' file info

'**********************************************************************

$CODE SEG       "lngfnam_text"

'**********************************************************************

FUNCTION LfnAlias (BYVAL Lfn AS STRING) PUBLIC AS STRING
'----------------------------------------------------------------------
' This function returns the alias (short) name for a given long file
' or directory name. If long names are not supported by the system,
' or if the file or directory doesn't exist, the function returns the
' given name unaltered.

' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

IF LfnSupported2 THEN                   ' Long file names supported ?
  DIM Address1    AS STRING PTR
  DIM Address2    AS STRING PTR
  DIM Lfn0        AS STRING
  DIM Sfn         AS STRING

  Lfn0 = Lfn + CHR$(0)
  Sfn = STRING$(128, 0)                 ' %MAX_DOS_PATH should be enough,
                                        ' but according to Ralf Brown's list
                                        ' it could be 128.
  Address1 = STRPTR32(Lfn0)
  Address2 = STRPTR32(Sfn)

  ASM   MOV     AX, &h7160              ; Get short name
  ASM   MOV     CX, &h8001              ; &h80: no expansion of subst drives
                                        ' &h01: get short name
  ASM   PUSH    DS
  ASM   PUSH    ES
  ASM   PUSH    SI
  ASM   PUSH    DI
  ASM   LDS     SI, Address1
  ASM   LES     DI, Address2
  ASM   INT     &h21
  ASM   POP     DI
  ASM   POP     SI
  ASM   POP     ES
  ASM   POP     DS
  ASM   JC      L50                     ; if error carry set and error no in AX
  ASM   XOR     AX, AX                  ; otherwise AX = 0
L50:
  ASM   MOV     DOSError, AX            ; DOS error code

  IF DOSError = 0 THEN
    FUNCTION = RTRIM$(Sfn, CHR$(0))
  ELSE
    FUNCTION = Lfn
  END IF
ELSE
  FUNCTION = Lfn
END IF
END FUNCTION

'**********************************************************************

FUNCTION LfnAliasCreate (BYVAL Lfn AS STRING) PUBLIC AS STRING
'----------------------------------------------------------------------
' This function returns the alias (short) name for a given long file
' name. However, if the file doesn't exist but its path does it is
' created. This way the functions simulates the conduct of PowerBasic's
' OPEN command for OUTPUT, APPEND, RANDOM or BINARY, and its BLOAD
' command.

' If the file's path doesn't exist or if the file can't be created the
' function returns the given name unaltered.

' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

LfnDirFirst Lfn, %FILE_ATTRIBUTE_ALL
SELECT CASE DOSError
CASE %NO_ERROR
  FUNCTION = LfnALias(Lfn)              ' Get alias name if file exists
CASE %ERROR_FILE_NOT_FOUND
  DIM Address AS STRING PTR
  DIM Handle AS WORD
  DIM Lfn0 AS STRING
  DOSError = 0
  Lfn0 = Lfn + CHR$(0)
  Address = STRPTR32(Lfn0)
  !     PUSH    SI                      ; Save data segment and pointers
  !     PUSH    DI
  !     PUSH    DS
  IF LfnSupported2 THEN                 ' Long file names supported ?
    !   MOV     AX, &h716C              ; Win95 function create or open file
    !   MOV     BX, %OPEN_ACCESS_READWRITE + %OPEN_SHARE_COMPATIBLE
    !   MOV     DX, %FILE_CREATE
    !   LDS     SI, Address             ; adres van bestandspecificatie
    !   XOR     DI, DI                  ; no alias hint
  ELSE
    !   MOV     AH, &h5B                ; DOS function create file
    !   LDS     DX, Address             ; adres van bestandspecificatie
  END IF
  !     MOV     CX, %FILE_ATTRIBUTE_NORMAL
  !     INT     &h21
  !     POP     DS                      ; Restore data segment and pointers
  !     POP     DI
  !     POP     SI
  !     JC      L51                     ; if error carry set and error no in AX
  !     MOV     Handle, AX
  !     XOR     AX, AX                  ; otherwise AX = 0
L51:
  !     MOV     DOSError, AX            ; DOS error code
  IF DOSError = %NO_ERROR THEN
    !   MOV     AH, &h3E                ; Function close file
    !   MOV     BX, Handle
    !   INT     &h21
    FUNCTION = LfnALias(Lfn)            ' Get alias name if file created
  ELSE
    FUNCTION = Lfn                      ' Return name unaltered if not created
  END IF
CASE ELSE
  FUNCTION = Lfn                        ' Return name unaltered if path not found
END SELECT
END FUNCTION

'**********************************************************************

FUNCTION LfnChDir (BYVAL Directory AS STRING) PUBLIC AS INTEGER
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's CHDIR statement.
' The function returns %TRUE (-1) if all went well, and %FALSE (0)
' otherwise.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

DIM Address     AS STRING PTR

Directory = Directory + CHR$(0)
Address = STRPTR32(Directory)

IF LfnSupported2 THEN                   ' Long file names supported ?
  ASM   MOV     AX, &h713B              ; Win95 function set actual directory
ELSE
  ASM   MOV     AH, &h3B                ; DOS function set actual directory
END IF
ASM     PUSH    DS
ASM     LDS     DX, Address             ; address of file name
ASM     INT     &h21
ASM     POP     DS
ASM     JC      L40                     ; if error carry set and error no in AX
ASM     XOR     AX, AX                  ; otherwise AX = 0
L40:
ASM     MOV     DOSError, AX            ; DOS error code

FUNCTION = (DOSError = 0)               ' return value

END FUNCTION

'**********************************************************************

FUNCTION LfnCurDir (BYVAL Drive AS STRING) PUBLIC AS STRING
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's CURDIR function.
' If an error is encountered the function returns "" as its value.
' Win95 returns the path all in caps.
'----------------------------------------------------------------------

DIM Buffer      AS STRING
DIM BufAddr     AS STRING PTR
DIM BufLen      AS INTEGER
DIM DriveNumber AS BYTE
DIM Lfn         AS INTEGER

IF LEN(Drive) = 0 THEN
  Drive = CurDrive
ELSE
  Drive = UCASE$(LEFT$(Drive$, 1))
END IF
DriveNumber = GetDriveNumber(Drive)
Lfn = LfnSupported2
IF Lfn THEN
  BufLen = %MAX_PATH
ELSE
  BufLen = %MAX_DOS_PATH
END IF
Buffer = STRING$(BufLen, 0)
BufAddr = STRPTR32(Buffer)

IF Lfn THEN                             ' Long file names supported ?
  ASM   MOV     AX, &h7147              ; Win95 function get current dir
ELSE
  ASM   MOV     AH, &h47                ; DOS function get current dir
END IF
ASM     PUSH    DS
ASM     PUSH    SI
ASM     LDS     SI, BufAddr             ; address of buffer in DS:SI
ASM     XOR     DH, DH
ASM     MOV     DL, DriveNumber         ; drive number in DX
ASM     INT     &h21
ASM     POP     SI
ASM     POP     DS
ASM     JC      L13                     ; if error carry set and error no in AX
ASM     XOR     AX, AX                  ; otherwise AX = 0
L13:
ASM     MOV     DOSError, AX            ; DOS error code

IF DOSError THEN
  FUNCTION = ""
ELSE
  FUNCTION = Drive + ":\" + RTRIM$(Buffer, CHR$(0)) ' the directory path
END IF

END FUNCTION

'**********************************************************************

FUNCTION LfnDirAlias () PUBLIC AS STRING
'----------------------------------------------------------------------
' Returns the alternate filename (alias) after a call to LfnDirFirst or
' LfnDirNext has been made.
' On it self it doesn't return a meaningfull result.
'----------------------------------------------------------------------
IF LfnSupported2 AND FindHandle <> 0 THEN
  FUNCTION = RTRIM$(FindData.cAlternateFileName, CHR$(0))
ELSE
  FUNCTION = ""
END IF
END FUNCTION

'**********************************************************************

FUNCTION LfnDirFirst (BYVAL Mask AS STRING, BYVAL Attr AS BYTE) PUBLIC _
         AS STRING
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's DIR function, first time.
' If an error is encountered the function returns "" as its value.
'----------------------------------------------------------------------

DIM PtrMask     AS STRING PTR

Mask = Mask + CHR$(0)
PtrMask = STRPTR32(Mask)

IF LfnSupported2 THEN

  ' Close a search handle not yet closed

  IF FindHandle THEN
    FindClose
  END IF

  PtrFindData = VARPTR32(FindData)

  ' Clean the data structure

  FindData.cFileName = STRING$(%MAX_PATH, 0)
  FindData.cAlternateFileName = STRING$(14, 0)

  ASM   MOV     AX, &h714E
  ASM   MOV     CL, Attr                        ; search attributes
  ASM   TEST    CL, %FILE_ATTRIBUTE_VOLUME      ; volume attribute included?
  ASM   JZ      L20                             ; then

  ASM   MOV     CH, %FILE_ATTRIBUTE_VOLUME      ; only volume names match
  ASM   JMP     SHORT L21

  L20:                                          ' otherwise

  ASM   OR      CL, %FILE_ATTRIBUTE_READONLY    ; include read-only files
  ASM                                           ; as does PB's DIR$
  ASM   MOV     CH, %FILE_ATTRIBUTE_NORMAL      ; and all files match

  L21:

  ASM   PUSH    DS                      ; preserve registers
  ASM   PUSH    SI
  ASM   PUSH    DI
  ASM   MOV     SI, 1                   ; MS-DOS date- and time format
  ASM   LES     DI, PtrFindData         ; Address of Win32_Find_Data structure
  ASM                                   ; receiving the file info
  ASM   LDS     DX, PtrMask             ; address of file mask
  ASM   INT     &h21
  ASM   POP     DI
  ASM   POP     SI
  ASM   POP     DS
  ASM   JC      L22                     ; everything OK?
  ASM   MOV     FindHandle, AX          ; then save search handle
  ASM   XOR     AX, AX                  ; no error

  L22:

  ASM   MOV     DOSError, AX            ; DOS error code

  IF DOSError THEN
    FindCLose                           ' close the find procedure
    FUNCTION = ""
  ELSE
    FUNCTION = RTRIM$(FindData.cFileName, CHR$(0))    ' the file name
  END IF

ELSE
  GetDTA  				' Get addres of DTA
  @DTA.fiFileName = STRING$(13, 0)	' Clean the data structure

  ASM   MOV     AH, &h4E
  ASM   MOV     CL, Attr                        ; search attributes
  ASM   OR      CL, %FILE_ATTRIBUTE_READONLY    ; include read-only files
  ASM                                           ; as does PB's DIR$
  ASM   XOR     CH, CH

  ASM   PUSH    DS                      ; preserve registers
  ASM   LDS     DX, PtrMask             ; address of file mask
  ASM   INT     &h21
  ASM   POP     DS
  ASM   JC      L23                     ; everything OK?
  ASM   XOR     AX, AX                  ; no error

  L23:

  ASM   MOV     DOSError, AX            ; DOS error code

  IF DOSError THEN
    FUNCTION = ""
  ELSE
    FUNCTION = RTRIM$(@DTA.fiFileName, CHR$(0))    ' the file name
  END IF

END IF

END FUNCTION

'**********************************************************************

FUNCTION LfnDirNext () PUBLIC AS STRING
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's DIR function, next times.
' If an error is encountered the function returns "" as its value.
'----------------------------------------------------------------------

IF LfnSupported2 THEN
  ' Clean the data structure

  FindData.cFileName = STRING$(%MAX_PATH, 0)
  FindData.cAlternateFileName = STRING$(14, 0)

  ASM   MOV     AX, &h714F
  ASM   MOV     BX, FindHandle
  ASM   PUSH    SI                      ; preserve registers
  ASM   PUSH    DI
  ASM   MOV     SI, 1                   ; MS-DOS date- and time format
  ASM   LES     DI, PtrFindData         ; Address of Win32_Find_Data structure
  ASM                                   ; receiving the file info
  ASM   INT     &h21
  ASM   POP     DI
  ASM   POP     SI
  ASM   JC      L24                     ; everything OK?
  ASM   XOR     AX, AX                  ; signal no error

  L24:                                  ' if error

  ASM   MOV     DOSError, AX            ; DOS error code

  IF DOSError THEN
    FindCLose                           ' close the find procedure
    FUNCTION = ""
  ELSE
    FUNCTION = RTRIM$(FindData.cFileName, CHR$(0))    ' the file name
  END IF
ELSE
  GetDTA  				' Get addres of DTA
  @DTA.fiFileName = STRING$(13, 0)	' Clean the data structure

  ASM   MOV     AH, &h4F
  ASM   INT     &h21
  ASM   JC      L25                     ; everything OK?
  ASM   XOR     AX, AX                  ; no error

  L25:

  ASM   MOV     DOSError, AX            ; DOS error code

  IF DOSError THEN
    FUNCTION = ""
  ELSE
    FUNCTION = RTRIM$(@DTA.fiFileName, CHR$(0))    ' the file name
  END IF

END IF
END FUNCTION

'**********************************************************************

SUB LfnFiles () PUBLIC
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's FILES statement
'----------------------------------------------------------------------
DIM col		AS INTEGER
DIM File	AS STRING

IF LfnSupported2 THEN
  File$ = LfnDirFirst$("*.*", &h10)
  DO WHILE LEN(File$)
    PRINT LfnDirAlias$;
    IF (LfnGetAttrib(File$) AND &h10) THEN
      PRINT TAB(14); "<DIR>";
    END IF
    PRINT TAB(22); File$
    File$ = LfnDirNext$
  LOOP
ELSE
  col = 1
  File$ = LfnDirFirst$("*.*", &h10)
  DO WHILE LEN(File$)
    IF col > 80 THEN
      PRINT
      col = 1
    END IF
    PRINT TAB(col); File$;
    IF (LfnGetAttrib(File$) AND &h10) THEN
      PRINT TAB(col + 13); "<DIR>";
    END IF
    INCR col, 20
    File$ = LfnDirNext$
  LOOP
  PRINT
END IF
PRINT USING$(STRING$(10, "#"), DiskFree("")); " Bytes free"

END SUB

'**********************************************************************

FUNCTION LfnGetAttrib (BYVAL File AS STRING) PUBLIC AS INTEGER
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's ATTRIB function.
' If an error is encountered the function returns -1 as its value.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

DIM Address     AS STRING PTR

FUNCTION = -1				' Initialize function value
File = File + CHR$(0)
Address = STRPTR32(File)

IF LfnSupported2 THEN                   ' Long file names supported ?
  ASM   MOV     AX, &h7143              ; Win95 function get or set attributes
  ASM   MOV     BL, 0                   ; action: retrieve attributes
ELSE
  ASM   MOV     AH, &h43                ; DOS function get or set attributes
  ASM   MOV     AL, 0                   ; action: retrieve attributes
END IF
ASM     PUSH    DS
ASM     LDS     DX, Address             ; address of file name
ASM     INT     &h21
ASM     POP     DS
ASM     JC      L11                     ; if error carry set and error no in AX
ASM     XOR     AX, AX                  ; otherwise AX = 0
ASM     MOV     FUNCTION, CX
L11:
ASM     MOV     DOSError, AX            ; DOS error code

END FUNCTION

'**********************************************************************

FUNCTION LfnKill (BYVAL File AS STRING) PUBLIC AS INTEGER
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's KILL statement.
' The function returns %TRUE (-1) if all went well, and %FALSE (0)
' otherwise.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

DIM Address     AS STRING PTR

File = File + CHR$(0)
Address = STRPTR32(File)

ASM     PUSH    DS
ASM     PUSH    SI
IF LfnSupported2 THEN                   ' Long file names supported ?
  ASM   MOV     SI, 1                   ; wildcards allowed
  ASM   MOV     CH, %FILE_ATTRIBUTE_NORMAL      ; must match attrib's
  ASM   MOV     CL, %FILE_ATTRIBUTE_NORMAL      ; search attrib's
  ASM   MOV     AX, &h7141              ; Win95 function delete file
ELSE
  ASM   MOV     AH, &h41                ; DOS function delete file
END IF
ASM     LDS     DX, Address             ; address of file name
ASM     INT     &h21
ASM     POP     SI
ASM     POP     DS
ASM     JC      L10                     ; if error carry set and error no in AX
ASM     XOR     AX, AX                  ; otherwise AX = 0
L10:
ASM     MOV     DOSError, AX            ; DOS error code

FUNCTION = (DOSError = 0)               ' return value

END FUNCTION

'**********************************************************************

FUNCTION LfnMkDir (BYVAL Directory AS STRING) PUBLIC AS INTEGER
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's MKDIR statement.
' The function returns %TRUE (-1) if all went well, and %FALSE (0)
' otherwise.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

DIM Address     AS STRING PTR

Directory = Directory + CHR$(0)
Address = STRPTR32(Directory)

IF LfnSupported2 THEN                   ' Long file names supported ?
  ASM   MOV     AX, &h7139              ; Win95 function make directory
ELSE
  ASM   MOV     AH, &h39                ; DOS function make directory
END IF
ASM     PUSH    DS
ASM     LDS     DX, Address             ; address of file name
ASM     INT     &h21
ASM     POP     DS
ASM     JC      L41                     ; if error carry set and error no in AX
ASM     XOR     AX, AX                  ; otherwise AX = 0
L41:
ASM     MOV     DOSError, AX            ; DOS error code

FUNCTION = (DOSError = 0)               ' return value

END FUNCTION

'**********************************************************************

FUNCTION LfnName (BYVAL OldName AS STRING, BYVAL NewName AS STRING) PUBLIC _
         AS INTEGER
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's NAME statement.
' The function returns %TRUE (-1) if all went well, and %FALSE (0)
' otherwise.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

DIM OldAddr     AS STRING PTR
DIM NewAddr     AS STRING PTR

OldName = OldName + CHR$(0)
NewName = NewName + CHR$(0)
OldAddr = STRPTR32(OldName)
NewAddr = STRPTR32(NewName)

IF LfnSupported2 THEN                   ' Long file names supported ?
  ASM   MOV     AX, &h7156              ; Win95 function rename file
ELSE
  ASM   MOV     AH, &h56                ; DOS function rename file
END IF
ASM     PUSH    DS
ASM     PUSH    DI
ASM     LES     DI, NewAddr             ; address of new name
ASM     LDS     DX, OldAddr             ; address of old name
ASM     INT     &h21
ASM     POP     DI
ASM     POP     DS
ASM     JC      L12                     ; if error carry set and error no in AX
ASM     XOR     AX, AX                  ; otherwise AX = 0
L12:
ASM     MOV     DOSError, AX            ; DOS error code

FUNCTION = (DOSError = 0)               ' return value

END FUNCTION

'**********************************************************************

FUNCTION LfnRmDir (BYVAL Directory AS STRING) PUBLIC AS INTEGER
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's RMDIR statement.
' The function returns %TRUE (-1) if all went well, and %FALSE (0)
' otherwise.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

DIM Address     AS STRING PTR

Directory = Directory + CHR$(0)
Address = STRPTR32(Directory)

IF LfnSupported2 THEN                   ' Long file names supported ?
  ASM   MOV     AX, &h713A              ; Win95 function remove directory
ELSE
  ASM   MOV     AH, &h3A                ; DOS function remove directory
END IF
ASM     PUSH    DS
ASM     LDS     DX, Address             ; address of file name
ASM     INT     &h21
ASM     POP     DS
ASM     JC      L42                     ; if error carry set and error no in AX
ASM     XOR     AX, AX                  ; otherwise AX = 0
L42:
ASM     MOV     DOSError, AX            ; DOS error code

FUNCTION = (DOSError = 0)               ' return value

END FUNCTION

'**********************************************************************

FUNCTION LfnSetAttrib (BYVAL File AS STRING, BYVAL Attribs AS WORD) _
  PUBLIC AS INTEGER
'----------------------------------------------------------------------
' Long file name replacement for PowerBasic's ATTRIB statement.
' The function returns %TRUE (-1) if all went well, and %FALSE (0)
' otherwise.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------

DIM Address     AS STRING PTR

FUNCTION = -1				' Initialize function value
File = File + CHR$(0)
Address = STRPTR32(File)

IF LfnSupported2 THEN                   ' Long file names supported ?
  ASM   MOV     AX, &h7143              ; Win95 function get or set attributes
  ASM   MOV     BL, 1                   ; action: set attributes
ELSE
  ASM   MOV     AH, &h43                ; DOS function get or set attributes
  ASM   MOV     AL, 1                   ; action: set attributes
END IF
ASM     MOV     CX, Attribs             ; the new attributes
ASM     PUSH    DS
ASM     LDS     DX, Address             ; address of file name
ASM     INT     &h21
ASM     POP     DS
ASM     JC      L14                     ; if error carry set and error no in AX
ASM     XOR     AX, AX                  ; otherwise AX = 0
ASM     MOV     FUNCTION, CX
L14:
ASM     MOV     DOSError, AX            ; DOS error code

FUNCTION = (DOSError = 0)               ' return value

END FUNCTION

'**********************************************************************

FUNCTION LfnSupported () PUBLIC AS INTEGER
'----------------------------------------------------------------------
' LfnSupported returns %TRUE (-1) if Windows 95 long file names are
' supported, and %FALSE (0) otherwise
' This is a PUBLIC wrapper for the PRIVATE sub LfnSupported2.
'----------------------------------------------------------------------
  LfnSupported = LfnSupported2
END FUNCTION

FUNCTION LfnSupported2 () PRIVATE AS INTEGER
'----------------------------------------------------------------------
' LfnSupported2 returns %TRUE (-1) if Windows 95 long file names are
' supported, and %FALSE (0) otherwise
'----------------------------------------------------------------------

DIM LfnSupport  AS STATIC INTEGER       ' flag telling if Long File Names
                                        ' are supported (1 = supported,
                                        ' -1 = not supported)

' Test if function was called already.
' If so, result is in flag LfnSupport

IF LfnSupport = 0 THEN
  IF LfnSupported3 THEN
    LfnSupport = 1
  ELSE
    LfnSupport = -1
  END IF
END IF
FUNCTION = (LfnSupport > 0)

END FUNCTION

FUNCTION LfnSupported3 () PRIVATE AS INTEGER
'----------------------------------------------------------------------
' LfnSupported3 returns %TRUE (-1) if Windows 95 long file names are
' supported, and %FALSE (0) otherwise
'----------------------------------------------------------------------

DIM DOSVersion  AS BYTE                 ' main DOS version number
DIM SystemName  AS STRING
DIM RootName    AS STRING
DIM LenSysName  AS INTEGER
DIM PtrSysName  AS STRING PTR
DIM PtrRootName AS STRING PTR

  'Initialize function value and flag

  FUNCTION = %FALSE

  'Do we have DOS 7 or higher (but not a DOS box under OS/2)?
  'N.B.: OS/2 version 2's DOS box has version number 20

  ASM   XOR     BX, BX                  ;Zero BX
  ASM   MOV     AX, &h3306              ;get DOS version (5 or higher)
  ASM   INT     &h21
  ASM   MOV     DOSVersion, BL

  IF DOSVersion < 7 OR DOSVersion >= 20 THEN

    ' If DOSVersion is 5 or 6 or 0 (DOS 4 or lower), or 20 or higher
    ' (OS/2), then no long Win95 names

    EXIT FUNCTION
  END IF

  'Do we have MS-DOS 7?

  ASM   PUSH    DS
  ASM   MOV     AX, &h4A33              ;function 4A33h
  ASM   INT     &h2F                    ;of ISR 2Fh
  ASM   POP     DS
  ASM   CMP     AX, 0                   ;is AX zeroed?
  ASM   JE      L2                      ;if not, no MS-DOS 7
    EXIT FUNCTION
  L2:

  'Does system support long names?

  SystemName$ = STRING$(8,0)
  PtrSysName  = STRPTR32(SystemName$)
  LenSysName  = LEN(SystemName$)
  RootName$   = "C:\" + CHR$(0)
  PtrRootName = STRPTR32(RootName$)

  ASM   PUSH    DS
  ASM   PUSH    DI
  ASM   LES     DI, PtrSysName          ;get address of SystemName in ES:DI
  ASM   MOV     CX, LenSysName          ;   and length in CX
  ASM   LDS     DX, PtrRootName         ;get address of RootName in DS:DX
  ASM   MOV     AX, &h71A0              ;get volume information
  ASM   INT     &h21
  ASM   POP     DI
  ASM   POP     DS
  ASM   JC      L3                      ;if OK,
    FUNCTION = %TRUE                    'then long names are supported
  L3:

END FUNCTION

'**********************************************************************

FUNCTION CurDrive () PUBLIC AS STRING
DIM DriveNumber AS BYTE

ASM     MOV     AH, &h19                ; get current disk drive
ASM     INT     &h21
ASM     INC     AL
ASM     MOV     DriveNumber, AL

FUNCTION = CHR$(DriveNumber + 64)
END FUNCTION

'**********************************************************************

FUNCTION DiskFree (BYVAL Drive AS STRING) PUBLIC AS DWORD
'----------------------------------------------------------------------
' Returns the number of free bytes on the given drive.
' If the given drive is invalid, the function returns 0, and DOSError
' has the value 15 (invalid disk drive).
'----------------------------------------------------------------------

DIM DriveNumber         AS BYTE
DIM BytesInSector       AS WORD
DIM SectorsInCluster    AS WORD
DIM FreeClusters        AS WORD

DriveNumber = GetDriveNumber(Drive)

ASM     MOV     AH, &h36
ASM     MOV     DL, DriveNumber
ASM     INT     &h21
ASM     CMP     AX, -1                  ; invalid disk drive if AX = -1
ASM     JE      L30
ASM     MOV     BytesInSector, CX
ASM     MOV     SectorsInCluster, AX
ASM     MOV     FreeClusters, BX

FUNCTION = CDWD(FreeClusters) * SectorsInCluster * BytesInSector
DOSError = 0
EXIT FUNCTION

L30:
FUNCTION = 0
DOSError = 15                           ' invalid disk drive

END FUNCTION

'**********************************************************************

FUNCTION DOSErr () PUBLIC AS INTEGER
'----------------------------------------------------------------------
' Returns the DOS error code
'----------------------------------------------------------------------
  FUNCTION = DOSError
END FUNCTION

'**********************************************************************

FUNCTION DOSErrTest () PUBLIC AS INTEGER
'----------------------------------------------------------------------
' Returns the DOS error code, and resets it to zero
'----------------------------------------------------------------------
  FUNCTION = DOSError
  DOSError = 0
END FUNCTION

'**********************************************************************

FUNCTION GetDTA () PUBLIC AS DWORD
'----------------------------------------------------------------------
' Returns a pointer to the DOS FileInfo structure
'----------------------------------------------------------------------
ASM     MOV     AH, &h2F
ASM     INT     &h21
ASM     MOV     WORD PTR DTA, BX
ASM     MOV     WORD PTR DTA[2], ES
FUNCTION = DTA
END FUNCTION

'**********************************************************************

FUNCTION GetPtrFindData () PUBLIC AS DWORD
'----------------------------------------------------------------------
' Returns a pointer to the Win32 FindData structure
'----------------------------------------------------------------------
GetPtrFindData = PtrFindData
END FUNCTION

'**********************************************************************

SUB FindCLose () PRIVATE
'----------------------------------------------------------------------
' Closes the long filename find procedure and frees its internal storage.
'----------------------------------------------------------------------

ASM     MOV     AX, &h71A1              ; close find procedure
ASM     MOV     BX, FindHandle
ASM     INT     &h21

FindHandle = 0

END SUB

'**********************************************************************

FUNCTION GetDriveNumber (Drive AS STRING) PRIVATE AS BYTE
'----------------------------------------------------------------------
' Sets drive number for given drive:
' A or a = 1, B or b = 2 etc.
' If Drive is an empty string the drive number is 0 to designate the
' current drive. If the first character of Drive isn't a letter, the
' drive number is 255 to designate an invalid drive.
'----------------------------------------------------------------------
DIM DriveNumber AS INTEGER
DriveNumber = ASCII(Drive)
SELECT CASE DriveNumber
CASE -1
  DriveNumber = 0
CASE 65 TO 90
  DECR DriveNumber, 64
CASE 97 TO 122
  DECR DriveNumber, 96
CASE ELSE
  DriveNumber = 255
END SELECT
FUNCTION = DriveNumber
END FUNCTION
